local HavenLootsServer = LibStub("AceAddon-3.0"):NewAddon("HavenLootsServer", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "AceTimer-3.0");
local mainTexture = "Interface\\AddOns\\HavenLoots\\Textures\\white.tga";
local mainFont = "Fonts\\FRIZQT__.TTF";

--##########################################################################
-- class variables
local sendType = "RAID";
local clearBox = OhaiyoClasses_BasicFrame.create();
local vCheckData = {};

--##########################################################################
-- checkers
local function IsMasterLooter()
	local _, _, ID = GetLootMethod();
	if ID == nil or UnitName("raid" ..ID) ~= UnitName("player") then
		return false;
	end
	return true;
end

--##########################################################################
-- frame movement functions
local function MouseDownClearBox(self, button)
	if button == "RightButton" then
		self:StartMoving();
		self.isMoving = true;
	elseif button == "LeftButton" then
		SendAddonMessage("HavenLoots", "clear", sendType);
	end
end

local function MouseUpClearBox(self, button)
	if self.isMoving then
		self:StopMovingOrSizing();
		self.isMoving = false;
		HavenLootsServer_point, HavenLootsServer_parent, HavenLootsServer_relativeTo, HavenLootsServer_x, HavenLootsServer_y = self:GetPoint();
	end
end

--##########################################################################
-- send loot table
function HavenLootsServer:LOOT_OPENED()
	if IsMasterLooter() ~= true then return end
	clearBox:Show();
end
HavenLootsServer:RegisterEvent("LOOT_OPENED");

function HavenLootsServer:LOOT_CLOSED()
	clearBox:Hide();
end
HavenLootsServer:RegisterEvent("LOOT_CLOSED");

local function SendNewItem()
	local link = GetLootSlotLink(LootFrame.selectedSlot);
	if link == nil then return end
	HavenLootsServer:SendCommMessage("HavenLoots", "new" .."_" ..link, sendType);
end

function HavenLootsServer:OPEN_MASTER_LOOT_LIST()
	if IsMasterLooter() ~= true then return end
	local info = UIDropDownMenu_CreateInfo();
	info.text = "HL Roll";
	info.notCheckable = 1;
	info.func = SendNewItem;
	UIDropDownMenu_AddButton(info);
end
HavenLootsServer:RegisterEvent("OPEN_MASTER_LOOT_LIST");

local function IsAlt(name)
	words = {}
	for word in name:gmatch("([^-]+)") do table.insert(words, word) end
	for i = 1, table.getn(HavenLootsServer_alts) do
		if HavenLootsServer_alts[i] == words[1] then
			return true;
		end
	end
	return false;
end

--##########################################################################
-- loading the addon
local function CreateClearBox()
	clearBox:PositionFrames("TOPLEFT", LootFrame, "BOTTOMLEFT", 170, 30, 0, 0);
	clearBox:SetTexture(mainTexture, 0.2, 0.2, 0.2, 0.5);
	clearBox:SetBackground(mainTexture, 0, 0, 0, 0.7);
	clearBox:SetFont(mainFont, 15, 1, 1, 1, 1);
	clearBox:SetText("Reset");
	clearBox:Hide();

	clearBox:GetFrame():EnableMouse(true);
	clearBox:GetFrame():RegisterForDrag("LeftButtonDown", "RightButtonDown");
	clearBox:GetFrame():SetMovable(true);
	clearBox:GetFrame():SetScript("OnMouseDown", MouseDownClearBox);
	clearBox:GetFrame():SetScript("OnMouseUp", MouseUpClearBox);
end

--##########################################################################
-- add loot rolls
local function AddLootRoll(itemLink, name, rollType)
	if IsAlt(name) then
		if rollType < 3 then
			rollType = rollType + 1;
		end
	end
	HavenLootsServer:SendCommMessage("HavenLoots", "res" .."_" ..name .."_"..rollType .."_" ..math.random(100) + ((5 - rollType) * 100) .."_" ..itemLink, sendType);
end

--##########################################################################
-- recieve messages
local function IsAlt(name)
	words = {}
	for word in name:gmatch("([^-]+)") do table.insert(words, word) end
	for i = 1, table.getn(HavenLootsServer_alts) do
		if HavenLootsServer_alts[i] == words[1] then
			return true;
		end
	end
	return false;
end

function HavenLootsServer:OnCommReceived(prefix, message, channel, sender)
	words = {}
	for word in message:gmatch("([^_]+)") do table.insert(words, word) end
	if IsMasterLooter() ~= true then return end
	if words[1] == "roll" then
		local itemLink = words[2];
		local rollType = tonumber(words[3]);
		AddLootRoll(itemLink, sender, rollType);
	elseif words[1] == "ver" then
		senderShorts = {}
		for senderShort in sender:gmatch("([^-]+)") do table.insert(senderShorts, senderShort) end
		local dataInsert = {
			name = senderShorts[1],
			ver = words[2],
		};
		table.insert(vCheckData, dataInsert);
	end
end
HavenLootsServer:RegisterComm("HavenLoots");

--##########################################################################
-- set alts
local function NameIsInRaid(input)
	for i = 1, 40 do
		local name = GetRaidRosterInfo(i);
		if name == input then
			return true;
		end
	end
	return false;
end

local function TakeFromAlt(input)
	for i = 1, table.getn(HavenLootsServer_alts) do
		if HavenLootsServer_alts[i] == input then
			table.remove(HavenLootsServer_alts, i);
			return true;
		end
	end
	return false;
end

HavenLootsServer:RegisterChatCommand("hls", "OnSlash");
HavenLootsServer:RegisterChatCommand("havenlootsserver", "OnSlash");
function HavenLootsServer:OnSlash(msg, editbox)
	if IsMasterLooter() ~= true then return end
	words = {}
	for word in msg:gmatch("[^%s]+") do table.insert(words, word) end
	if words[1] == "alt" then
		table.insert(HavenLootsServer_alts, GetUnitName("target", false));
		SendChatMessage("[Haven Loots] You have been marked as an alt! This means you will be given a lower priority on loot rolls. Roll as if you were a main and the addon will do the rest. Whisper me if this is incorrect!", "WHISPER", "Common", GetUnitName("target", true));
		DEFAULT_CHAT_FRAME:AddMessage("[HavenLoots]: " ..GetUnitName("target", false) .." added to alts.", 1, 1, 0);
	elseif words[1] == "altremove" then
		local success = TakeFromAlt(GetUnitName("target", false));
		if success then
			SendChatMessage("[Haven Loots] You are no longer marked as an alt. Lucky you!", "WHISPER", "Common", GetUnitName("target", true));
			DEFAULT_CHAT_FRAME:AddMessage("[HavenLoots]: " ..GetUnitName("target", false) .." cleared from alts.", 1, 1, 0);
		else
			DEFAULT_CHAT_FRAME:AddMessage("[HavenLoots]: Error! No one in the current list of alts under that name.", 1, 0, 0);
		end
	elseif words[1] == "altclear" then
		alts = {};
		DEFAULT_CHAT_FRAME:AddMessage("[HavenLoots]: Alts Cleared", 1, 1, 0);
		SendChatMessage("[Haven Loots] All alts have been reset! If you were one before you aren't now.", "RAID", "Common");
	elseif words[1] == "check" then
		SendChatMessage("[Haven Loots] Sending a version check.", "RAID", "Common");
		HavenLootsServer:SendCommMessage("HavenLoots", "check", sendType);
		HavenLootsServer:ScheduleTimer("VersionChecker", 2);
	elseif words[1] == "clear" then
		HavenLootsServer:SendCommMessage("HavenLoots", "clear", sendType);
	end
end

--##########################################################################
-- loading
function HavenLootsServer:OnInitialize()
	if HavenLootsServer_alts == nil then HavenLootsServer_alts = {} end
	CreateClearBox();
end

--##########################################################################
-- handling the version check
function HavenLootsServer:VersionChecker()
	local latestVersion = 0;
	local latestVersionString = "";
	local latestNames = {};
	local notLatest = {};
	local notInstalled = {};

	for i = 1, table.getn(vCheckData) do
		if tonumber(vCheckData[i].ver) > latestVersion then
			latestVersion = tonumber(vCheckData[i].ver);
			latestVersionString = vCheckData[i].ver;
		end
	end

	for i = 1, table.getn(vCheckData) do
		if tonumber(vCheckData[i].ver) == latestVersion then
			table.insert(latestNames, vCheckData[i].name);
		else
			table.insert(notLatest, vCheckData[i]);
		end
	end

	for i = 1, GetNumGroupMembers() do
		local name = GetUnitName("raid" ..i, false);
		local inlist = false;
		for x = 1, table.getn(vCheckData) do
			if vCheckData[i].name == name then
				inlist = true;
			end
		end
		if inlist ~= true then
			table.insert(notInstalled, name);
		end
	end

	local latestNamesString = "[Haven Loots] Latest Version (" ..latestVersionString.. "): ";
	for i = 1, table.getn(latestNames) do
		latestNamesString = latestNamesString ..latestNames[i] .." ";
	end
	DEFAULT_CHAT_FRAME:AddMessage(latestNamesString, 0, 1, 0);

	local outofDateString = "[Haven Loots] Out of date: ";
	for i = 1, table.getn(notLatest) do
		outofDateString = outofDateString ..notLatest[i].name .." (" ..notLatest[i].ver ..") ";
	end
	DEFAULT_CHAT_FRAME:AddMessage(outofDateString, 1, 1, 0);

	local notInstalledString = "[Haven Loots] Not installed: "
	for i = 1, table.getn(notInstalled) do
		notInstalledString = notInstalledString ..notInstalled[i] .." ";
	end
	DEFAULT_CHAT_FRAME:AddMessage(notInstalledString, 1, 0, 0);
	vCheckData = {};
end